@@ -5,10 +5,13 @@ Abstract base class for jobs
 import abc
 import asyncio
 import collections
-import json
 import os
+import pickle
 import re
 
+import unidecode
+
+from .. import constants
 from ..utils import cached_property, fs, signal
 
 import logging  # isort:skip
@@ -19,12 +22,16 @@ class JobBase(abc.ABC):
     """
     Base class for all jobs
 
-    :param str home_directory: Directory that is used to store files and cache
-        output
+    :param str home_directory: Directory that is used to store created files
+    :param str cache_directory: Directory that is used to cache output
     :param str ignore_cache: Whether cached output and previously created files
         should not be re-used
     :param bool hidden: Whether to hide the job's output in the UI
     :param bool autostart: Whether this job is started automatically
+    :param condition: Callable that gets no arguments and returns whether this
+        job is enabled or disabled
+    :param callbacks: Mapping of :attr:`signal` names to callable or sequence of
+        callables to :meth:`~.signal.Signal.register` for that signal
 
     Any additional keyword arguments are passed on to :meth:`initialize`.
 
@@ -50,13 +57,16 @@ class JobBase(abc.ABC):
 
     @property
     def home_directory(self):
-        """
-        Directory that is used to store files
-
-        Cached output is stored in a subdirectory called ".output".
-        """
+        """Directory that is used to store files, e.g. generated images"""
         return self._home_directory
 
+    @cached_property
+    def cache_directory(self):
+        """Path to existing directory that stores :attr:`cache_file`"""
+        if not os.path.exists(self._cache_directory):
+            os.makedirs(self._cache_directory)
+        return self._cache_directory
+
     @property
     def ignore_cache(self):
         """Whether cached output and previously created files should not be re-used"""
@@ -79,6 +89,31 @@ class JobBase(abc.ABC):
         """
         return self._autostart
 
+    @property
+    def condition(self):
+        """Callable that gets no arguments and is called by :attr:`is_enabled`"""
+        return self._condition
+
+    @condition.setter
+    def condition(self, function):
+        if not callable(function):
+            raise TypeError(f'Not callable: {function!r}')
+        self._condition = function
+
+    @property
+    def is_enabled(self):
+        """
+        Return value of `condition` as :class:`bool`
+
+        If this value is `False`, this job must not be started nor included in
+        the UI.
+
+        This property must be checked by the UI every time any job finishes.
+        This job must be started and added to the UI if it changes from `False`
+        to `True`.
+        """
+        return bool(self._condition())
+
     @property
     def kwargs(self):
         """Keyword arguments from instantiation as :class:`dict`"""
@@ -94,46 +129,65 @@ class JobBase(abc.ABC):
 
         ``finished``
             is emitted when :meth:`finish` is called or when output is read from
-            cache. Registered callbacks get no arguments.
+            cache. Registered callbacks get the job instance as a positional
+            argument.
 
         ``output``
             is emitted when :meth:`send` is called or when output is read from
             cache. Registered callbacks get the value passed to :meth:`send` as
             a positional argument.
 
+        ``info``
+            is emitted when :attr:`info` is set. Registered callbacks get the
+            new :attr:`info` as a positional argument.
+
+        ``warning``
+            is emitted when :meth:`warn` is called. Registered callbacks get the
+            value passed to :meth:`warn` as a positional argument.
+
         ``error``
             is emitted when :meth:`error` is called. Registered callbacks get
             the value passed to :meth:`error` as a positional argument.
         """
         return self._signal
 
-    def __init__(self, *, home_directory=None, ignore_cache=False, hidden=False,
-                 autostart=True, **kwargs):
-        if home_directory:
-            # Custome home specific to some content. Store cache in hidden
-            # directory in there.
-            self._home_directory = str(home_directory)
-            self._cache_directory = os.path.join(self._home_directory, '.cache')
-        else:
-            # Default home, something like /tmp/upsies. We want to store files
-            # and cache in the same directory.
-            self._home_directory = fs.tmpdir()
-            self._cache_directory = self.home_directory
+    def __init__(self, *, home_directory=None, cache_directory=None,
+                 ignore_cache=False, hidden=False, autostart=True,
+                 condition=None, callbacks={}, **kwargs):
+        self._home_directory = home_directory if home_directory else '.'
+        self._cache_directory = cache_directory if cache_directory else constants.CACHE_DIRPATH
         self._ignore_cache = bool(ignore_cache)
         self._hidden = bool(hidden)
         self._autostart = bool(autostart)
+        self.condition = condition or (lambda: True)
         self._is_started = False
+        self._is_executed = False
         self._exception = None
-        self._errors = []
         self._output = []
-        self._signal = signal.Signal('output', 'error', 'finished')
+        self._warnings = []
+        self._errors = []
+        self._info = ''
+        self._tasks = []
+        self._finished_event = asyncio.Event()
+
+        self._signal = signal.Signal('output', 'info', 'warning', 'error', 'finished')
         self._signal.register('output', lambda output: self._output.append(str(output)))
+        self._signal.register('warning', lambda warning: self._warnings.append(warning))
         self._signal.register('error', lambda error: self._errors.append(error))
         self._signal.record('output')
-        self._finished_event = asyncio.Event()
+
         self._kwargs = kwargs
         self.initialize(**kwargs)
 
+        # Add signal callbacks after `initialize` had the chance to add custom
+        # signals.
+        for signal_name, callback in callbacks.items():
+            if callable(callback):
+                self._signal.register(signal_name, callback)
+            else:
+                for cb in callback:
+                    self._signal.register(signal_name, cb)
+
     def initialize(self):
         """
         Called by :meth:`__init__` with additional keyword arguments
@@ -143,7 +197,7 @@ class JobBase(abc.ABC):
 
     def execute(self):
         """
-        Do the job, e.g. prompt for user input or start background worker
+        Do the job, e.g. ask for user input or start a task or subprocess
 
         This method must not block.
         """
@@ -158,22 +212,29 @@ class JobBase(abc.ABC):
         :raise RuntimeError: if this method is called multiple times or if
             reading from cache file fails unexpectedly
         """
-        if self._is_started:
-            raise RuntimeError('start() was already called')
-        else:
-            self._is_started = True
+        if self.is_enabled:
+            if self._is_started:
+                raise RuntimeError('start() was already called')
+            else:
+                self._is_started = True
 
-        try:
-            cache_was_read = self._read_cache()
-        except BaseException:
-            self.finish()
-            raise
+            try:
+                cache_was_read = self._read_cache()
+            except BaseException:
+                self.finish()
+                raise
 
-        if cache_was_read:
-            self.finish()
-        else:
-            _log.debug('Executing %r', self)
-            self.execute()
+            if cache_was_read:
+                self.finish()
+            else:
+                _log.debug('Executing job: %s', self.name)
+                self._is_executed = True
+                self.execute()
+
+    @property
+    def is_started(self):
+        """Whether :meth:`start` was called"""
+        return self._is_started
 
     async def wait(self):
         """
@@ -191,11 +252,11 @@ class JobBase(abc.ABC):
 
         Calling this method multiple times simultaneously is safe.
 
-        :raise: Any exceptions given to :meth:`exception`
+        :raise: The first exception given to :meth:`exception`
         """
         await self._finished_event.wait()
-        if self._exception is not None:
-            raise self._exception
+        if self.raised:
+            raise self.raised
 
     def finish(self):
         """
@@ -205,15 +266,18 @@ class JobBase(abc.ABC):
 
         :attr:`is_finished` is `True` after this method was called.
 
-        Calling this method multiple times simultaneously is safe.
+        Calling this method from multiple coroutines simultaneously is safe.
 
         Subclasses must call the parent's method (``super().finish()``).
 
         This method does not block.
         """
         if not self.is_finished:
+            for task in self._tasks:
+                if not task.done():
+                    task.cancel()
             self._finished_event.set()
-            self.signal.emit('finished')
+            self.signal.emit('finished', self)
             self._write_cache()
 
     @property
@@ -225,11 +289,16 @@ class JobBase(abc.ABC):
     def exit_code(self):
         """`0` if job was successful, `> 0` otherwise, None while job is not finished"""
         if self.is_finished:
-            if not self.output or self.errors or self._exception:
+            if not self.output or self.errors or self.raised:
                 return 1
             else:
                 return 0
 
+    def send(self, output):
+        """Append `output` to :attr:`output` and emit ``output`` signal"""
+        if not self.is_finished:
+            self.signal.emit('output', str(output))
+
     @property
     def output(self):
         """Result of this job as an immutable sequence of strings"""
@@ -238,26 +307,36 @@ class JobBase(abc.ABC):
     @property
     def info(self):
         """
-        Additional information that is only displayed in the UI and not part of the
-        job's result
-        """
-        return ''
+        String that is only displayed while the job is running and not part of the
+        job's output
 
-    def send(self, output):
-        """Append `output` to :attr:`output` and emit ``output`` signal"""
+        Setting this property emits the ``info`` signal.
+        """
+        return self._info
+
+    @info.setter
+    def info(self, info):
+        self._info = str(info)
+        self.signal.emit('info', info)
+
+    def warn(self, warning):
+        """Append `warning` to :attr:`warnings` and emit ``warning`` signal"""
         if not self.is_finished:
-            if output:
-                self.signal.emit('output', str(output))
+            self.signal.emit('warning', str(warning))
 
     @property
-    def errors(self):
-        """Sequence of reported errors (strings or exceptions)"""
-        return tuple(self._errors)
+    def warnings(self):
+        """
+        Sequence of non-critical error messages the user can override or resolve
 
-    def clear_errors(self):
-        """Empty :attr:`errors`"""
+        Unlike :attr:`errors`, warnings do not imply failure by default.
+        """
+        return tuple(self._warnings)
+
+    def clear_warnings(self):
+        """Empty :attr:`warnings`"""
         if not self.is_finished:
-            self._errors.clear()
+            self._warnings.clear()
 
     def error(self, error, finish=False):
         """
@@ -271,12 +350,21 @@ class JobBase(abc.ABC):
                 self.finish()
             self.signal.emit('error', error)
 
+    @property
+    def errors(self):
+        """
+        Sequence of critical errors (strings or exceptions)
+
+        By default, :attr:`exit_code` is non-zero if any errors were reported.
+        """
+        return tuple(self._errors)
+
     def exception(self, exception):
         """
-        Set exception to raise in :meth:`wait` and call :meth:`finish`
+        Make `exception` available as :attr:`raised` and call :meth:`finish`
 
         .. warning:: Setting an exception means you want to throw a traceback in
-                     the user's face, which may not be a good idea.
+                     the user's face.
 
         :param Exception exception: Exception instance
         """
@@ -288,22 +376,75 @@ class JobBase(abc.ABC):
             self._exception = exception
             self.finish()
 
+    @property
+    def raised(self):
+        """Exception passed to :meth:`exception`"""
+        return self._exception
+
+    def add_task(self, coro, callback=None, finish_when_done=False):
+        """
+        Run asynchronous coroutine in a background task and return immediately
+
+        This method should be used to call coroutine functions and other
+        awaitables in a synchronous context.
+
+        Any exceptions from `coro` are passed to :meth:`exception`.
+        :class:`asyncio.CancelledError` is ignored.
+
+        :param coro: Any awaitable object
+        :param callback: Callable that is called with the return value of `coro`
+        :param bool finish_when_done: Whether to call :meth:`finish` when `coro`
+            is done
+
+        :return: :class:`asyncio.Task` instance
+        """
+
+        async def wrapper(coro, callback, finish_when_done):
+            try:
+                result = await asyncio.ensure_future(coro)
+            except asyncio.CancelledError:
+                pass
+            except BaseException as e:
+                _log.debug('%s: Caught %r from %r', self.name, e, coro)
+                self.exception(e)
+            else:
+                if callback:
+                    callback(result)
+                return result
+            finally:
+                if finish_when_done:
+                    _log.debug('Finishing %r because of finished coro: %r', self.name, coro)
+                    self.finish()
+
+        async def catch_cancelled_error(coro):
+            try:
+                return await asyncio.ensure_future(coro)
+            except asyncio.CancelledError:
+                _log.debug('%s: Cancelled: %r', self.name, coro)
+
+        wrapped = asyncio.ensure_future(
+            catch_cancelled_error(
+                wrapper(coro, callback, finish_when_done),
+            ),
+        )
+        self._tasks.append(wrapped)
+        return wrapped
+
     def _write_cache(self):
         """
-        Store :attr:`output` in :attr:`cache_file`
+        Store recorded signals in :attr:`cache_file`
 
-        The base class implementation stores output as JSON. Child classes may
-        use other formats.
+        Emitted signals are serialized with :meth:`_serialize_for_cache`.
 
-        :raise RuntimeError: if :attr:`output` is not JSON-encodable or
-            :attr:`cache_file` is not writable
+        :raise RuntimeError: if writing :attr:`cache_file` fails
         """
-        if self.output and self.exit_code == 0 and self.cache_file:
-            emissions_string = json.dumps(self.signal.emissions, indent=4)
+        if self.signal.emissions and self.exit_code == 0 and self.cache_file and self._is_executed:
+            emissions_serialized = self._serialize_for_cache(self.signal.emissions)
+            _log.debug('%s: Caching emitted signals: %r', self.name, self.signal.emissions)
             try:
-                with open(self.cache_file, 'w') as f:
-                    f.write(emissions_string)
-                    f.write('\n')
+                with open(self.cache_file, 'wb') as f:
+                    f.write(emissions_serialized)
+                    f.write(b'\n')
             except OSError as e:
                 if e.strerror:
                     raise RuntimeError(f'Unable to write cache {self.cache_file}: {e.strerror}')
@@ -312,7 +453,9 @@ class JobBase(abc.ABC):
 
     def _read_cache(self):
         """
-        Set :attr:`output` to data stored in :attr:`cache_file`
+        Read cached :attr:`~.signal.Signal.emissions` from :attr:`cache_file`
+
+        Emitted signals are deserialized with :meth:`_deserialize_from_cache`.
 
         :raise RuntimeError: if :attr:`cache_file` exists and is unreadable
 
@@ -320,26 +463,42 @@ class JobBase(abc.ABC):
         """
         if not self._ignore_cache and self.cache_file and os.path.exists(self.cache_file):
             try:
-                with open(self.cache_file, 'r') as f:
-                    emissions_string = f.read()
+                with open(self.cache_file, 'rb') as f:
+                    emissions_serialized = f.read()
             except OSError as e:
                 if e.strerror:
                     raise RuntimeError(f'Unable to read cache {self.cache_file}: {e.strerror}')
                 else:
                     raise RuntimeError(f'Unable to read cache {self.cache_file}: {e}')
             else:
-                emissions = json.loads(emissions_string)
-                _log.debug('%s: Replaying cached signals: %r', self.name, emissions)
-                self.signal.replay(emissions)
-                return True
+                emissions_deserialized = self._deserialize_from_cache(emissions_serialized)
+                _log.debug('%s: Replaying cached signals: %r', self.name, emissions_deserialized)
+                if emissions_deserialized:
+                    self.signal.replay(emissions_deserialized)
+                    return True
         return False
 
-    @cached_property
-    def cache_directory(self):
-        """Path to existing directory that stores cache files"""
-        if not os.path.exists(self._cache_directory):
-            os.mkdir(self._cache_directory)
-        return self._cache_directory
+    def _serialize_for_cache(self, emissions):
+        """
+        Convert emitted signals to cache format
+
+        :param emissions: See :attr:`Signal.emissions`
+
+        :return: :class:`bytes`
+        """
+        return pickle.dumps(emissions, protocol=0, fix_imports=False)
+
+    def _deserialize_from_cache(self, emissions_serialized):
+        """
+        Convert return value of :meth:`_serialize_for_cache` back to emitted signals
+
+        :param emissions_serialized: :class:`bytes` object
+
+        :return: See :attr:`Signal.emissions`
+        """
+        return pickle.loads(emissions_serialized)
+
+    _max_filename_len = 255
 
     @cached_property
     def cache_file(self):
@@ -354,21 +513,17 @@ class JobBase(abc.ABC):
         elif not cache_id:
             filename = f'{self.name}.json'
         else:
-            # Avoid file name being too long. 255 bytes seems common. Leave
-            # some headroom for multibytes.
+            # Avoid file name being too long. 255 bytes seems common.
             # https://en.wikipedia.org/wiki/Comparison_of_file_systems#Limits
-            max_len = 250 - len(self.name) - len('..json')
-            cache_id_str = self._cache_data_as_string(cache_id)
+            max_len = self._max_filename_len - len(self.name) - len('..json')
+            cache_id_str = self._cache_id_as_string(cache_id)
             if len(cache_id_str) > max_len:
                 cache_id_str = ''.join((
                     cache_id_str[:int(max_len / 2 - 1)],
                     '…',
                     cache_id_str[-int(max_len / 2 - 1):],
                 ))
-            if cache_id_str:
-                filename = f'{self.name}.{cache_id_str}.json'
-            else:
-                filename = f'{self.name}.json'
+            filename = f'{self.name}.{cache_id_str}.json'
             filename = fs.sanitize_filename(filename)
         return os.path.join(self.cache_directory, filename)
 
@@ -377,37 +532,39 @@ class JobBase(abc.ABC):
         """
         Unique object based on the job's input data
 
-        The return value is turned into a string. Items of non-string iterables
-        are passed to :class:`str` and :meth:`~.str.join`\\ ed with ",".
+        The return value is turned into a string. If it is a non-string sequence
+        or a mapping, items are converted to strings and joined with ",".
+        Multibyte characters and directory delimiters are replaced.
 
-        If this property returns `None`, cache is not read or written.
+        If this property returns `None`, :attr:`cache_file` is not read or
+        written.
 
-        The default implementation uses the arguments passed to
-        :meth:`initialize`.
+        If this property returns any other falsy value, :attr:`name` is used.
         """
-        # Check if any values don't have a string representation to prevent
-        # random cache IDs
-        no_str_regex = re.compile(r'^<[\w\.]+ object at 0x[a-f\d]+>$')
-        for key, value in self.kwargs.items():
-            if no_str_regex.search(str(key)):
-                raise RuntimeError(f'{type(key)!r} has no string representation')
-            elif no_str_regex.search(str(value)):
-                raise RuntimeError(f'{type(value)!r} has no string representation')
-        return self._kwargs
+        return ''
 
-    def _cache_data_as_string(self, value):
+    def _cache_id_as_string(self, value):
         if isinstance(value, collections.abc.Mapping):
-            return ','.join((f'{k}={self._cache_data_as_string(v)}' for k, v in value.items()))
-
+            return ','.join((f'{self._cache_id_value_as_string(k)}={self._cache_id_as_string(v)}'
+                             for k, v in value.items()))
         elif isinstance(value, collections.abc.Iterable) and not isinstance(value, str):
-            return ','.join((self._cache_data_as_string(v) for v in value))
-
+            return ','.join((self._cache_id_as_string(v) for v in value))
         elif isinstance(value, (str, os.PathLike)) and os.path.exists(value):
             # Use same cache file for absolute and relative paths
             return str(os.path.realpath(value))
-
         else:
-            return str(value)
+            return self._cache_id_value_as_string(value)
+
+    _object_without_str_regex = re.compile(r'^<.*>$')
+
+    def _cache_id_value_as_string(self, value):
+        # Avoid multibyte characters stay below maximum file length
+        value_string = unidecode.unidecode(str(value))
+        # Check if `value` has a proper string representation to prevent random
+        # cache IDs. We don't want "<foo.bar object at 0x...>" in our cache ID.
+        if self._object_without_str_regex.search(value_string):
+            raise RuntimeError(f'{type(value)!r} has no string representation')
+        return value_string
 
 
 class QueueJobBase(JobBase):
@@ -443,16 +600,18 @@ class QueueJobBase(JobBase):
     async def _read_queue(self):
         while True:
             value = await self._queue.get()
-            if self.is_finished:
-                raise RuntimeError(f'{type(self).__name__} is already finished')
-            elif value is None:
+            if value is None or self.is_finished:
                 break
             else:
-                await self._handle_input(value)
+                try:
+                    await self.handle_input(value)
+                except BaseException as e:
+                    self.exception(e)
+                    break
         self.finish()
 
     @abc.abstractmethod
-    async def _handle_input(self, value):
+    async def handle_input(self, value):
         """Handle `value` from queue"""
 
     def enqueue(self, value):
