@@ -32,20 +32,20 @@ class BbTrackerJobs(TrackerJobsBase):
 
     @property
     def is_movie_release(self):
-        return self.release_type_job.choice is release.ReleaseType.movie
+        return self.release_type is release.ReleaseType.movie
 
     @property
     def is_season_release(self):
-        return self.release_type_job.choice is release.ReleaseType.season
+        return self.release_type is release.ReleaseType.season
 
     @property
     def is_episode_release(self):
-        return self.release_type_job.choice is release.ReleaseType.episode
+        return self.release_type is release.ReleaseType.episode
 
     @property
     def is_series_release(self):
-        return self.release_type_job.choice in (release.ReleaseType.season,
-                                                release.ReleaseType.episode)
+        return self.release_type in (release.ReleaseType.season,
+                                     release.ReleaseType.episode)
 
     @property
     def season(self):
@@ -120,6 +120,9 @@ class BbTrackerJobs(TrackerJobsBase):
                 result = await result_getter(id)
                 if result:
                     return result
+        # Default to return value from final webdb, e.g. empty string, empty
+        # list, etc.
+        return result
 
     # Jobs
 
@@ -239,8 +242,8 @@ class BbTrackerJobs(TrackerJobsBase):
 
     @cached_property
     def imdb_job(self):
-        """:class:`~.jobs.webdb.SearchWebDbJob` instance"""
-        return jobs.webdb.SearchWebDbJob(
+        """:class:`~.jobs.webdb.WebDbSearchJob` instance"""
+        return jobs.webdb.WebDbSearchJob(
             condition=self.make_job_condition('imdb_job', release.ReleaseType.movie),
             content_path=self.content_path,
             db=self.imdb,
@@ -249,7 +252,7 @@ class BbTrackerJobs(TrackerJobsBase):
 
     @cached_property
     def movie_title_job(self):
-        self.imdb_job.signal.register('output', self.movie_title_imdb_id_handler)
+        self.imdb_job.signal.register('output', self.fill_in_movie_title)
         return jobs.dialog.TextFieldJob(
             name='movie-title',
             label='Title',
@@ -263,7 +266,7 @@ class BbTrackerJobs(TrackerJobsBase):
         if not text:
             raise ValueError(f'Invalid title: {text}')
 
-    def movie_title_imdb_id_handler(self, imdb_id):
+    def fill_in_movie_title(self, imdb_id):
         default_text = self.release_name.title_with_aka
         coro = self.get_movie_title(imdb_id)
         task = self.movie_title_job.fetch_text(
@@ -275,7 +278,7 @@ class BbTrackerJobs(TrackerJobsBase):
 
     @cached_property
     def movie_year_job(self):
-        self.imdb_job.signal.register('output', self.movie_year_imdb_id_handler)
+        self.imdb_job.signal.register('output', self.fill_in_movie_year)
         return jobs.dialog.TextFieldJob(
             name='movie-year',
             label='Year',
@@ -289,7 +292,7 @@ class BbTrackerJobs(TrackerJobsBase):
         # Raises ValueError if not a valid year
         self.release_name.year = text
 
-    def movie_year_imdb_id_handler(self, imdb_id):
+    def fill_in_movie_year(self, imdb_id):
         default_text = self.release_name.year
         coro = self.imdb.year(imdb_id)
         task = self.movie_year_job.fetch_text(
@@ -439,11 +442,11 @@ class BbTrackerJobs(TrackerJobsBase):
         )
 
     async def movie_get_poster_url(self, poster_job):
-        return await self.get_poster_url(poster_job, self.get_movie_poster_url)
+        return await self.get_resized_poster_url(poster_job, self.get_movie_poster_url)
 
     @cached_property
     def movie_tags_job(self):
-        self.imdb_job.signal.register('output', self.movie_tags_imdb_id_handler)
+        self.imdb_job.signal.register('output', self.fill_in_movie_tags)
         return jobs.dialog.TextFieldJob(
             name='movie-tags',
             label='Tags',
@@ -457,8 +460,8 @@ class BbTrackerJobs(TrackerJobsBase):
         if not text:
             raise ValueError(f'Invalid tags: {text}')
 
-    def movie_tags_imdb_id_handler(self, imdb_id):
-        coro = self.get_tags(self.imdb, imdb_id)
+    def fill_in_movie_tags(self, imdb_id):
+        coro = self.get_tags()
         task = self.movie_tags_job.fetch_text(
             coro=coro,
             finish_on_success=True,
@@ -493,8 +496,8 @@ class BbTrackerJobs(TrackerJobsBase):
 
     @cached_property
     def tvmaze_job(self):
-        """:class:`~.jobs.webdb.SearchWebDbJob` instance"""
-        return jobs.webdb.SearchWebDbJob(
+        """:class:`~.jobs.webdb.WebDbSearchJob` instance"""
+        return jobs.webdb.WebDbSearchJob(
             condition=self.make_job_condition('tvmaze_job', release.ReleaseType.season, release.ReleaseType.episode),
             content_path=self.content_path,
             db=self.tvmaze,
@@ -503,7 +506,7 @@ class BbTrackerJobs(TrackerJobsBase):
 
     @cached_property
     def series_title_job(self):
-        self.tvmaze_job.signal.register('output', self.series_title_tvmaze_id_handler)
+        self.tvmaze_job.signal.register('output', self.fill_in_series_title)
         return jobs.dialog.TextFieldJob(
             name='series-title',
             label='Title',
@@ -520,7 +523,7 @@ class BbTrackerJobs(TrackerJobsBase):
         if unknown:
             raise ValueError(f'Failed to autodetect: {unknown}')
 
-    def series_title_tvmaze_id_handler(self, tvmaze_id):
+    def fill_in_series_title(self, tvmaze_id):
         coro = self.get_series_title_and_release_info(tvmaze_id)
         default_text = self.release_name.title_with_aka_and_year
         task = self.series_title_job.fetch_text(
@@ -542,11 +545,11 @@ class BbTrackerJobs(TrackerJobsBase):
         )
 
     async def series_get_poster_url(self, poster_job):
-        return await self.get_poster_url(poster_job, self.get_series_poster_url)
+        return await self.get_resized_poster_url(poster_job, self.get_series_poster_url)
 
     @cached_property
     def series_tags_job(self):
-        self.tvmaze_job.signal.register('output', self.series_tags_tvmaze_id_handler)
+        self.tvmaze_job.signal.register('output', self.fill_in_series_tags)
         return jobs.dialog.TextFieldJob(
             name='series-tags',
             label='Tags',
@@ -560,8 +563,8 @@ class BbTrackerJobs(TrackerJobsBase):
         if not text:
             raise ValueError(f'Invalid tags: {text}')
 
-    def series_tags_tvmaze_id_handler(self, tvmaze_id):
-        coro = self.get_tags(self.tvmaze, tvmaze_id)
+    def fill_in_series_tags(self, tvmaze_id):
+        coro = self.get_tags()
         task = self.series_tags_job.fetch_text(
             coro=coro,
             finish_on_success=True,
@@ -612,7 +615,7 @@ class BbTrackerJobs(TrackerJobsBase):
         if self.release_name.source == 'WEBRip':
             return 'WebRip'
         else:
-            return self.release_name.source
+            return string.remove_suffix(self.release_name.source, 'Remux').strip()
 
     @property
     def release_info_remux(self):
@@ -758,18 +761,36 @@ class BbTrackerJobs(TrackerJobsBase):
         elif self.is_episode_release:
             title.append(str(self.release_name.episodes))
 
-        # "[Source / VideoCodec / AudioCodec / Container / Resolution( / ...)]"
         info = [
-            self.release_info_remux,
+            # [Source / VideoCodec / AudioCodec / Container / Resolution]
             self.release_info_source,
             self.release_name.video_format,
-            self.release_info_10bit,
             self.release_info_audio_format,
             fs.file_extension(video.first_video(self.content_path)).upper(),
+            self.release_info_resolution,
+
+            # Scene tags
             self.release_info_proper,
             self.release_info_repack,
-            self.release_info_resolution,
+
+            # Special formats
+            self.release_info_remux,
             self.release_info_hdr10,
+            self.release_info_10bit,
+
+            # Editions
+            self.release_info_uncensored,
+            self.release_info_uncut,
+            self.release_info_unrated,
+            self.release_info_remastered,
+            self.release_info_directors_cut,
+            self.release_info_extended_edition,
+            self.release_info_anniversary_edition,
+            self.release_info_criterion_edition,
+            self.release_info_special_edition,
+            self.release_info_limited_edition,
+
+            # Features
             self.release_info_dual_audio,
             self.release_info_commentary,
             self.release_info_subtitles,
@@ -777,12 +798,44 @@ class BbTrackerJobs(TrackerJobsBase):
         info_string = ' / '.join(i for i in info if i)
         return ' '.join(title) + f' [{info_string}]'
 
-    async def get_poster_url(self, poster_job, poster_url_getter):
-        # Get original poster URL (e.g. "http://imdb.com/...jpg")
-        poster_url = await poster_url_getter()
-        if not poster_url:
-            self.error('Failed to find poster')
+    async def get_resized_poster_url(self, poster_job, poster_url_getter):
+        poster_path = await self.get_poster_file(poster_job, poster_url_getter)
+        if not poster_path:
+            self.error('Provide a poster file or URL with the --poster-file option.')
         else:
+            # Resize poster
+            try:
+                resized_poster_path = image.resize(poster_path, width=300)
+            except errors.ImageResizeError as e:
+                self.error(f'Poster resizing failed: {e}')
+            else:
+                _log.debug('Poster resized: %r', resized_poster_path)
+                # Upload poster to self.image_host
+                poster_job.info = f'Uploading poster to {self.image_host.name}'
+                try:
+                    return await self.image_host.upload(resized_poster_path)
+                except errors.RequestError as e:
+                    self.error(f'Poster upload failed: {e}')
+                finally:
+                    poster_job.info = ''
+
+    async def get_poster_file(self, poster_job, poster_url_getter):
+        if self.cli_args.poster_file:
+            # Get poster from CLI argument
+            if re.search(r'^[a-z]+://', self.cli_args.poster_file):
+                # CLI argument is URL
+                poster_url = self.cli_args.poster_file
+            else:
+                # CLI argument is file path
+                return self.cli_args.poster_file
+        else:
+            # Get poster URL from webdb (e.g. "https://imdb.com/...jpg")
+            poster_url = await poster_url_getter()
+
+        if not poster_url:
+            self.error('Failed to find poster URL.')
+        else:
+            # Download poster
             poster_job.info = f'Downloading poster: {poster_url}'
             poster_path = os.path.join(poster_job.home_directory, 'poster.bb.jpg')
             try:
@@ -790,25 +843,7 @@ class BbTrackerJobs(TrackerJobsBase):
             except errors.RequestError as e:
                 self.error(f'Poster download failed: {e}')
             else:
-                if not os.path.exists(poster_path) or not os.path.getsize(poster_path) > 0:
-                    self.error(f'Poster download failed: {poster_url}')
-                else:
-                    # Resize poster
-                    try:
-                        resized_poster_path = image.resize(poster_path, width=300)
-                    except errors.ImageResizeError as e:
-                        self.error(f'Poster resizing failed: {e}')
-                    else:
-                        _log.debug('Poster resized: %r', resized_poster_path)
-                        # Upload poster to self.image_host
-                        poster_job.info = f'Uploading poster to {self.image_host.name}'
-                        try:
-                            real_poster_url = await self.image_host.upload(resized_poster_path)
-                        except errors.RequestError as e:
-                            self.error(f'Poster upload failed: {e}')
-                        else:
-                            poster_job.info = ''
-                            poster_job.send(real_poster_url)
+                return poster_path
 
     async def get_movie_poster_url(self):
         imdb_id = await self.get_imdb_id()
@@ -826,10 +861,17 @@ class BbTrackerJobs(TrackerJobsBase):
 
     def get_movie_release_info(self):
         info = (
-            self.release_info_576p_PAL,
-            self.release_info_remux,
+            # Scene tags
             self.release_info_proper,
             self.release_info_repack,
+
+            # Special formats
+            self.release_info_remux,
+            self.release_info_576p_PAL,
+            self.release_info_hdr10,
+            self.release_info_10bit,
+
+            # Editions
             self.release_info_uncensored,
             self.release_info_uncut,
             self.release_info_unrated,
@@ -840,15 +882,15 @@ class BbTrackerJobs(TrackerJobsBase):
             self.release_info_criterion_edition,
             self.release_info_special_edition,
             self.release_info_limited_edition,
+
+            # Features
             self.release_info_dual_audio,
-            self.release_info_hdr10,
-            self.release_info_10bit,
             self.release_info_commentary,
             self.release_info_subtitles,
         )
         return ' / '.join(i for i in info if i)
 
-    async def get_tags(self, webdb, id):
+    async def get_tags(self):
         def normalize_tags(strings):
             normalized = []
             for s in strings:
@@ -871,13 +913,17 @@ class BbTrackerJobs(TrackerJobsBase):
                 for item in seq
             )
 
+        # For movies, TVmaze ID is None and tvmaze is ignored.
+        # For series, IMDb might have information TVmaze is missing.
+        webdbs = (self.tvmaze, self.imdb)
+
         # Gather tags
-        tags = list(await webdb.keywords(id))
+        tags = list(await self.try_webdbs(webdbs, 'genres'))
         if self.is_movie_release:
-            tags.extend(await webdb.directors(id))
+            tags.extend(await self.try_webdbs(webdbs, 'directors'))
         elif self.is_series_release:
-            tags.extend(await webdb.creators(id))
-        tags.extend(await webdb.cast(id))
+            tags.extend(await self.try_webdbs(webdbs, 'creators'))
+        tags.extend(await self.try_webdbs(webdbs, 'cast'))
 
         # Replace spaces, non-ASCII characters, etc
         tags = normalize_tags(tags)
@@ -1144,7 +1190,7 @@ class BbTrackerJobs(TrackerJobsBase):
             return post_data
 
         else:
-            raise RuntimeError(f'Weird release type: {self.release_type_job.choice!r}')
+            raise RuntimeError(f'Weird release type: {self.release_type!r}')
 
     @property
     def post_data_screenshot_urls(self):
