@@ -11,11 +11,22 @@ class webdb_search(CommandBase):
     """
     Search online database like IMDb to get an ID
 
-    Pressing ``Enter`` searches for the current query. Pressing ``Enter`` again without
-    changing the query selects the focused search result.
+    Pressing ``Enter`` searches for the current query. Pressing ``Enter`` again
+    without changing the query selects the focused search result.
 
     The focused search result can be opened in the default web browser with
     ``Alt-Enter``.
+
+    Search results can be narrowed down with the following parameters:
+
+      - year:YYYY
+        Only return results with a specific release year.
+
+      - type:series|movie
+        Only return movies or series.
+
+      - id:ID
+        Search for a specific, known ID.
     """
 
     names = ('id',)
@@ -35,7 +46,7 @@ class webdb_search(CommandBase):
     @utils.cached_property
     def jobs(self):
         return (
-            jobs.webdb.SearchWebDbJob(
+            jobs.webdb.WebDbSearchJob(
                 home_directory=self.home_directory,
                 cache_directory=self.cache_directory,
                 ignore_cache=self.args.ignore_cache,
