@@ -17,9 +17,10 @@ class Query:
     Search query for databases like IMDb
 
     :param str title: Name of the movie or TV series
-    :param str type: :class:`~.types.ReleaseType` enum or one of its value names
+    :param type: :class:`~.types.ReleaseType` enum or one of its value names
     :param year: Year of release
     :type year: str or int
+    :param str id: Known ID for a specific DB
 
     :raise ValueError: if an invalid argument is passed
     """
@@ -28,24 +29,38 @@ class Query:
     def _normalize_title(title):
         return ' '.join(title.casefold().strip().split())
 
-    def __init__(self, title, year=None, type=ReleaseType.unknown):
-        self._type = ReleaseType(type)
-        self._title = str(title)
-        self._title_normalized = self._normalize_title(self._title)
-        self._year = None if year is None else str(year)
-        if self._year is not None and not 1800 < int(self._year) < 2100:
-            raise ValueError(f'Invalid year: {self._year}')
+    _kwarg_defaults = {
+        'year': None,
+        'type': ReleaseType.unknown,
+        'id': None,
+    }
+
+    def __init__(self, title='', **kwargs):
+        self.title = title
+        self.type = kwargs.get('type', self._kwarg_defaults['type'])
+        self.year = kwargs.get('year', self._kwarg_defaults['year'])
+        self.id = kwargs.get('id', self._kwarg_defaults['id'])
+        self._title_normalized = self._normalize_title(self.title)
+        self._kwargs_order = tuple(kwargs)
 
     @property
     def type(self):
         """:class:`~.types.ReleaseType` value"""
         return self._type
 
+    @type.setter
+    def type(self, type):
+        self._type = ReleaseType(type)
+
     @property
     def title(self):
         """Name of the movie or TV series"""
         return self._title
 
+    @title.setter
+    def title(self, title):
+        self._title = str(title)
+
     @property
     def title_normalized(self):
         """Same as :attr:`title` but in stripped lower case with deduplicated spaces"""
@@ -56,6 +71,30 @@ class Query:
         """Year of release"""
         return self._year
 
+    @year.setter
+    def year(self, year):
+        if year is None:
+            self._year = None
+        else:
+            try:
+                year_int = int(year)
+            except (TypeError, ValueError):
+                raise ValueError(f'Invalid year: {year}')
+            else:
+                if not 1800 < year_int < 2100:
+                    raise ValueError(f'Invalid year: {year}')
+                else:
+                    self._year = str(year_int)
+
+    @property
+    def id(self):
+        """Known ID for a specific DB"""
+        return self._id
+
+    @id.setter
+    def id(self, id):
+        self._id = None if id is None else str(id)
+
     _types = {
         ReleaseType.movie: ('movie', 'film'),
         ReleaseType.season: ('season', 'series', 'tv', 'show', 'tvshow'),
@@ -65,6 +104,7 @@ class Query:
     _kw_regex = {
         'year': r'year:(\d{4})',
         'type': rf'type:({"|".join(_known_types)})',
+        'id': r'id:(\S+)',
     }
 
     @classmethod
@@ -75,16 +115,11 @@ class Query:
         The returned :class:`Query` is case-insensitive and has any superfluous
         whitespace removed.
 
-        Keyword arguments are extracted by looking for ``"year:YEAR"`` and
-        ``"type:TYPE"`` in `query` where ``YEAR`` is a four-digit number and
-        ``TYPE`` is something like "movie", "film", "tv", etc.
+        Keyword arguments are extracted by looking for ``"year:YEAR"``,
+        ``"type:TYPE"`` and ``"id:ID"`` in `query` where ``YEAR`` is a
+        four-digit number, ``TYPE`` is something like "movie", "film", "tv", etc
+        and ``ID`` is a known ID for the DB this query is meant for.
         """
-        # Normalize query:
-        #   - Case-insensitive
-        #   - Remove leading/trailing white space
-        #   - Deduplicate white space
-        query = ' '.join(str(query).strip().split())
-
         def get_kwarg(string):
             for kw, regex in cls._kw_regex.items():
                 match = re.search(f'^{regex}$', part)
@@ -98,19 +133,22 @@ class Query:
                         return 'type', ReleaseType.episode
                     elif kw == 'year':
                         return 'year', value
+                    elif kw == 'id':
+                        return 'id', value
             return None, None
 
         # Extract "key:value" pairs (e.g. "year:2015")
         title = []
         kwargs = {}
-        for part in query.split():
+        for part in str(query).strip().split():
             kw, value = get_kwarg(part)
             if (kw, value) != (None, None):
                 kwargs[kw] = value
             else:
                 title.append(part)
-
-        return cls(title=' '.join(title), **kwargs)
+        if title:
+            kwargs['title'] = ' '.join(title)
+        return cls(**kwargs)
 
     @classmethod
     def from_path(cls, path):
@@ -134,25 +172,30 @@ class Query:
                 self.title_normalized == other.title_normalized
                 and self.year == other.year
                 and self.type is other.type
+                and self.id == other.id
             )
         else:
             return NotImplemented
 
     def __str__(self):
-        parts = [self.title]
-        if self.year:
-            parts.append(f'year:{self.year}')
-        if self.type is not ReleaseType.unknown:
-            parts.append(f'type:{self.type}')
-        return ' '.join(parts)
+        if self.id:
+            return f'id:{self.id}'
+        else:
+            parts = [self.title]
+            for attr in self._kwargs_order:
+                value = getattr(self, attr)
+                if value:
+                    parts.append(f'{attr}:{value}')
+            return ' '.join(parts)
 
     def __repr__(self):
         kwargs = ', '.join(
             f'{k}={v!r}'
             for k, v in (('title', self.title),
                          ('year', self.year),
-                         ('type', self.type))
-            if v not in (None, ReleaseType.unknown)
+                         ('type', self.type),
+                         ('id', self.id))
+            if v
         )
         return f'{type(self).__name__}({kwargs})'
 
@@ -163,16 +206,16 @@ class SearchResult:
 
     Keyword arguments are available as attributes.
 
-    `cast`, `countries`, `director`, `keywords`, `summary`, `title_english` and
+    `cast`, `countries`, `directors`, `genres`, `summary`, `title_english` and
     `title_original` may be coroutine functions that return the expected value.
 
-    :param str director: Name of director
+    :param str directors: Sequence of directors
     :param cast: Short list of actor names
     :type cast: sequence of str
     :param str countries: List of country names of origin
     :param str id: ID for the relevant DB
-    :param keywords: Short list of keywords, e.g. `["horror", "commedy"]`
-    :type keywords: sequence of str
+    :param genres: Short list of genres, e.g. `["horror", "commedy"]`
+    :type genres: sequence of str
     :param summary: Short text that describes the movie or series
     :param str title: Title of the movie or series
     :param str title_english: English title of the movie or series
@@ -182,8 +225,8 @@ class SearchResult:
     :param str year: Release year; for series this should be the year of the
         first airing of the first episode of the first season
     """
-    def __init__(self, *, id, type, url, year, cast=(), countries=(), director='',
-                 keywords=(), summary='', title, title_english='',
+    def __init__(self, *, id, type, url, year, cast=(), countries=(), directors='',
+                 genres=(), summary='', title, title_english='',
                  title_original=''):
         self._info = {
             'id' : id,
@@ -194,8 +237,8 @@ class SearchResult:
             # These may be coroutine functions
             'cast' : cast,
             'countries' : countries,
-            'director' : director,
-            'keywords' : keywords,
+            'directors' : directors,
+            'genres' : genres,
             'summary' : summary,
             'title_english' : title_english,
             'title_original' : title_original,
